/*
  This file is part of the KTextTemplate library

  SPDX-FileCopyrightText: 2009, 2010 Stephen Kelly <steveire@gmail.com>

  SPDX-License-Identifier: LGPL-2.1-or-later

*/

#ifndef KTEXTTEMPLATE_TAGS_P_H
#define KTEXTTEMPLATE_TAGS_P_H

namespace KTextTemplate
{

static const char BLOCK_TAG_START[] = "{%";
static const char BLOCK_TAG_END[] = "%}";
static const char VARIABLE_TAG_START[] = "{{";
static const char VARIABLE_TAG_END[] = "}}";
static const char COMMENT_TAG_START[] = "{#";
static const char COMMENT_TAG_END[] = "#}";
// static const char * SINGLE_BRACE_START = "{";
// static const char * SINGLE_BRACE_END = "}";
}

#endif
