
{% extends "shared/base.html" %}

{% block content %}
  {% rssfeed "http://wxdata.weather.com/wxdata/weather/rss/local/GMXX0007?cm_ven=LWO&amp;cm_cat=rss" "/rss/channel/item" %}
  <h2><a href="{% xmlrole "link/string()" %}">{% xmlrole "title/string()" %}</a></h2>
  {% xmlrole "description/string()" %}
  {% endrssfeed %}
{% endblock %}
