/*
  This file is part of the KTextTemplate library

  SPDX-FileCopyrightText: 2010 Stephen Kelly <steveire@gmail.com>

  SPDX-License-Identifier: LGPL-2.1-or-later

*/

QCoreApplication::translate("GR_FILENAME", "Birthday");
QCoreApplication::translate("GR_FILENAME", "%n People", 0, QCoreApplication::CodecForTr, count);
QCoreApplication::translate("GR_FILENAME", "Name", "Name of a Person");
QCoreApplication::translate("GR_FILENAME", "Name", "Name of a Book");
QCoreApplication::translate("GR_FILENAME", "%n People", "%n people are logged in", QCoreApplication::CodecForTr, count);
QCoreApplication::translate("GR_FILENAME", "%n file(s) copied to %1", 0, QCoreApplication::CodecForTr, count);
QCoreApplication::translate("GR_FILENAME", "%n to %1", "Files are being copied", QCoreApplication::CodecForTr, count);
QCoreApplication::translate("GR_FILENAME", "%n to %1", "Files have already been copied", QCoreApplication::CodecForTr, count);
QCoreApplication::translate("GR_FILENAME", "from %1 to %2", "Files are being copied from %1 to %2");
QCoreApplication::translate("GR_FILENAME", "%1 messages at %2, fraction of total: %3. Rating : %4");
QCoreApplication::translate("GR_FILENAME", "Today");
QCoreApplication::translate("GR_FILENAME", "Today is %1");
QCoreApplication::translate("GR_FILENAME", "Today &amp; tomorrow");
QCoreApplication::translate("GR_FILENAME", "%n people today", 0, QCoreApplication::CodecForTr, count);
QCoreApplication::translate("GR_FILENAME", "Today is", "The current day is");
QCoreApplication::translate("GR_FILENAME", "%n people visited today", "The number of people who have visited today", QCoreApplication::CodecForTr, count);
QCoreApplication::translate("GR_FILENAME", "Snowman: ☃");
